import mock
import unittest

from pastry.resources.databags import Databags


class DatabagsTestCase(unittest.TestCase):

    @mock.patch('pastry.resources.databags.Base.index', return_value='index')
    def test_index(self, base):
        self.assertEqual(Databags.index(), 'index')

    @mock.patch('pastry.resources.databags.Base.exists', return_value=True)
    def test_exists(self, base):
        self.assertEqual(Databags.exists('group'), True)

    @mock.patch('pastry.resources.databags.PastryClient.call', return_value='item')
    def test_get_item(self, call):
        self.assertEqual(Databags.get_item('databagid', 'itemid'), 'item')

    @mock.patch('pastry.resources.databags.PastryClient.call', return_value='item')
    def test_create_item(self, call):
        self.assertEqual(Databags.create_item('databagid', {}), 'item')

    @mock.patch('pastry.resources.databags.PastryClient.call', return_value='item')
    def test_delete_item(self, call):
        self.assertEqual(Databags.delete_item('databagid', 'itemid'), 'item')

    @mock.patch('pastry.resources.databags.PastryClient.call', return_value='item')
    def test_update_item(self, call):
        self.assertEqual(Databags.update_item('databagid', 'itemid', {}), 'item')



import mock
import unittest

from pastry.exceptions import HttpError

def raise_error():
    raise HttpError('message', 'statuscode')


class PastryClientTestCase(unittest.TestCase):


    def test_init(self):
        self.assertRaises(HttpError, raise_error)

    def test_str(self):
        error = HttpError('message', 'statuscode')
        self.assertEqual(str(error), '(statuscode) message')



'''Cookbook provides the methods for chef cookbooks'''

import requests

from .base import Base
from pastry.exceptions import HttpError
from pastry.pastry_client import PastryClient


class Cookbooks(Base):
    '''
    Provides methods for interacting with chef cookbooks
    '''
    _base_url = '/organizations/%(org)s/cookbooks'

    @classmethod
    def index(cls):
        '''
        Fetches all of the cookbooks (and versions) on the chef server

        :return: All the chef cookbooks and versions
        :rtype: hash
        '''
        return super(Cookbooks, cls).index()

    @classmethod
    def exists(cls, cookbook):
        '''
        Checks if a cookbook exists on the chef server

        :param cookbook: The Cookbook's name
        :type cookbook: string
        :return: If the cookbook exists
        :rtype: boolean
        '''
        return super(Cookbooks, cls).exists(cookbook)

    @classmethod
    def contents(cls, cookbook, version='_latest'):
        '''
        Fetches the cookbooks list of files that chef knows about

        :param cookbook: The cookbook's name
        :param version: The cookbook's version
        :type cookbook: string
        :type version: string
        :return: All of the files the cookbook knows about
        :rtype: hash
        '''
        return PastryClient.call(
            '%s/%s/%s' % (cls.base_url(), cookbook, version))

    @classmethod
    def parse_filename(cls, filename):
        '''
        Splits the file path so that it can be used to call the chef api

        :param filename: The file's path relative to the cookbook root
        :type filename: string
        :return: The type of file, specificity, and filename
        :rtype: iterable
        '''
        parts = filename.split('/')
        if len(parts) == 1:
            return ['root_files', 'default', filename]
        return parts

    @classmethod
    def file_content(cls, cookbook, filename, version='_latest'):
        '''
        Fetches the contents of a specific file in a cookbook

        :param cookbook: The cookbook's name
        :param filename: The name (and path) of the file to fetch
        :param version: The cookbook's version
        :type cookbook: string
        :type filename: string
        :type version: string
        :return: The raw contents of the specified file
        :rtype: string
        '''
        file_type, specificity, name = cls.parse_filename(filename)
        files = cls.contents(cookbook, version=version)
        for file_info in files[file_type]:
            if name == file_info['name'] and \
                    specificity == file_info['specificity']:
                resp = requests.get(
                    file_info['url'], verify=PastryClient.verify)
                if not resp.ok:
                    raise HttpError(resp.text, resp.status_code)
                return resp.text



import mock
import unittest

from pastry.exceptions import HttpError
from pastry.resources.cookbooks import Cookbooks


class CookbooksTestCase(unittest.TestCase):

    @mock.patch('pastry.resources.cookbooks.Base.index', return_value='index')
    def test_index(self, base):
        self.assertEqual(Cookbooks.index(), 'index')

    @mock.patch('pastry.resources.cookbooks.Base.exists', return_value=True)
    def test_exists(self, base):
        self.assertEqual(Cookbooks.exists('group'), True)

    @mock.patch('pastry.resources.cookbooks.PastryClient.call', return_value='contents')
    def test_contents(self, call):
        self.assertEqual(Cookbooks.contents('name', version='version'), 'contents')
        call.assert_called_with('/organizations/%(org)s/cookbooks/name/version')

    def test_parse_filename(self):
        self.assertEqual(['root_files', 'default', 'filename'], Cookbooks.parse_filename('filename'))
        self.assertEqual(
            ['filetype', 'specificity', 'filename'],
            Cookbooks.parse_filename('filetype/specificity/filename')
        )

    @mock.patch('pastry.resources.cookbooks.Cookbooks.parse_filename', return_value=[
        'filetype', 'specificity', 'filename'
    ])
    @mock.patch('pastry.resources.cookbooks.Cookbooks.contents', return_value={
        'filetype': [{
            'name': 'filename',
            'specificity': 'specificity',
            'url': 'url'
        }]
    })
    @mock.patch('pastry.resources.cookbooks.requests.get')
    def test_file_content(self, get, contents, parse):
        response = mock.MagicMock()
        response.text = 'contents'
        response.ok = False
        get.return_value = response
        self.assertRaises(HttpError, Cookbooks.file_content, 'cookbook', 'filename')
        response.ok = True
        self.assertEqual(Cookbooks.file_content('cookbook', 'filename'), response.text)



import mock
import unittest

from pastry.utils import auth


class AuthTestCase(unittest.TestCase):

    @mock.patch('pastry.utils.auth.encode', return_value='encoded')
    @mock.patch('pastry.utils.auth.sha1')
    def test_hashencode(self, sha1, encode):
        self.assertEqual(auth.hashencode('content'), 'encoded')
        sha1.assert_called_with('content')

    @mock.patch('pastry.utils.auth.b64encode')
    def test_encode(self, b64encode):
        expected = [
            u'abcdefghijabcdefghijabcdefghijabcdefghijabcdefghijabcdefghij',
            u'abcdefghijabcdefghijabcdefghijabcdefghijabcdefghijabcdefghij',
            u'abcdefghijabcdefghijabcdefghijabcdefghijabcdefghijabcdefghij',
            u'abcdefghijabcdefghijabcdefghijabcdefghijabcdefghijabcdefghij',
            u'abcdefghijabcdefghijabcdefghijabcdefghijabcdefghijabcdefghij'
        ]
        b64encode.return_value = ''.join('abcdefghij' * 30)
        self.assertEqual(auth.encode('hashed'), expected)

    @mock.patch('pastry.utils.auth.encode')
    @mock.patch('pastry.utils.auth.sign', return_value='signature')
    def test_authorization_headers(self, sign, encode):
        encode.return_value = ['part1', 'part2', 'part3']
        expected_result = {
            'X-ops-authorization-1': 'part1',
            'X-ops-authorization-2': 'part2',
            'X-ops-authorization-3': 'part3'
        }
        with mock.patch('__builtin__.open', mock.mock_open(read_data='key')):
            self.assertEqual(
                auth.authorization_headers('keypath', 'canonical_source'),
                expected_result
            )

    @mock.patch('pastry.utils.auth.authorization_headers', return_value={'auth':'headers'})
    @mock.patch('pastry.utils.auth.hashencode', return_value=['encoded'])
    @mock.patch('pastry.utils.auth.datetime')
    @mock.patch('pastry.utils.auth.json')
    def test_signed_headers(self, json, datetime, hashencode, authorization_headers):
        now = mock.MagicMock()
        now.strftime.return_value = 'timestamp'
        datetime.utcnow.return_value = now
        expected = {
            'Accept': 'application/json',
            'X-chef-version': '12.8.0',
            'X-ops-content-hash': 'encoded',
            'X-ops-sign': 'version=1.0',
            'X-ops-timestamp': 'timestamp',
            'X-ops-userid': 'client',
            'auth': 'headers'
        }
        auth.signed_headers('client', 'keypath', 'path')
        json.dumps.assert_not_called()
        self.assertEqual(
            auth.signed_headers('client', 'keypath', 'path', data='data'),
            expected
        )
        json.dumps.assert_called_with('data')



import mock
import unittest

from pastry.resources.base import Base
from pastry.exceptions import HttpError


class BaseTestCase(unittest.TestCase):

    def test_base_url(self):
        Base._base_url = 'url'
        self.assertEqual(Base.base_url(), 'url')
        Base._base_url = None
        self.assertRaises(ValueError, Base.base_url)

    @mock.patch('pastry.resources.base.Base.base_url', return_value='url')
    @mock.patch('pastry.resources.base.PastryClient')
    def test_index(self, pastry_client, base_url):
        pastry_client.call.return_value = 'result'
        self.assertEqual(Base.index(), 'result')
        pastry_client.call.assert_called_with('url')

    @mock.patch('pastry.resources.base.Base.base_url', return_value='url')
    @mock.patch('pastry.resources.base.PastryClient')
    def test_get(self, pastry_client, base_url):
        pastry_client.call.return_value = 'result'
        self.assertEqual(Base.get('instance'), 'result')
        pastry_client.call.assert_called_with('url/instance')

    @mock.patch('pastry.resources.base.Base.base_url', return_value='url')
    @mock.patch('pastry.resources.base.PastryClient')
    def test_create(self, pastry_client, base_url):
        pastry_client.call.return_value = 'result'
        self.assertEqual(Base.create({}), 'result')
        pastry_client.call.assert_called_with('url', method='POST', data={})

    @mock.patch('pastry.resources.base.Base.base_url', return_value='url')
    @mock.patch('pastry.resources.base.PastryClient')
    def test_update(self, pastry_client, base_url):
        pastry_client.call.return_value = 'result'
        self.assertEqual(Base.update('instance', {}), 'result')
        pastry_client.call.assert_called_with('url/instance', method='PUT', data={})

    @mock.patch('pastry.resources.base.Base.base_url', return_value='url')
    @mock.patch('pastry.resources.base.PastryClient')
    def test_delete(self, pastry_client, base_url):
        pastry_client.call.return_value = 'result'
        self.assertEqual(Base.delete('instanceid'), 'result')
        pastry_client.call.assert_called_with('url/instanceid', method='DELETE')

    @mock.patch('pastry.resources.base.Base.get')
    def test_exists(self, get):
        get.return_value = {'instanceid': 'id'}
        self.assertEqual(Base.exists('instanceid'), True)
        get.assert_called_with('instanceid')
        get.side_effect = HttpError('message', 404)
        self.assertEqual(Base.exists('instanceid'), False)

    def test_escape_query(self):
        self.assertEqual(
            Base.escape_query('test%query%string'),
            'test%%query%%string'
        )

    @mock.patch('pastry.resources.base.Base.base_url', return_value='url')
    @mock.patch('pastry.resources.base.PastryClient')
    def test_get_acl(self, pastry_client, base_url):
        pastry_client.call.return_value = 'result'
        self.assertEqual(Base.get_acl('instance'), 'result')
        pastry_client.call.assert_called_with('url/instance/_acl')

    @mock.patch('pastry.resources.base.Base.base_url', return_value='url')
    @mock.patch('pastry.resources.base.PastryClient')
    def test_set_permission(self, pastry_client, base_url):
        pastry_client.call.return_value = 'result'
        self.assertEqual(Base.set_permission('instance', 'permission', {}), 'result')
        pastry_client.call.assert_called_with(
            'url/instance/_acl/permission',
            method='PUT',
            data={'permission': {}}
        )



'''Nodes provides the methods for chef node resources'''

from .base import Base
from pastry.pastry_client import PastryClient


class Pushy(Base):
    '''
    Provides access to the chef pushy api
    '''
    _base_url = '/organizations/%(org)s/pushy'

    @classmethod
    def status(cls, nodename):
        '''
        Checks the push jobs status of a node

        :param nodename: The Node's nodename
        :type nodename: string
        :return: If pushy jobs is available on the node
        :rtype: boolean
        '''
        return PastryClient.call(
            '%s/node_states/%s' % (cls.base_url(), nodename))



'''Nodes provides the methods for chef node resources'''

from .base import Base


class Nodes(Base):
    '''
    Provides access to the chef nodes
    '''
    _base_url = '/organizations/%(org)s/nodes'

    @classmethod
    def exists(cls, nodename):
        '''
        Checks if a node exists on the chef server

        :param nodename: The Node's nodename
        :type nodename: string
        :return: If the node exists
        :rtype: boolean
        '''
        return super(Nodes, cls).exists(nodename)

    @classmethod
    def get_acl(cls, nodename):
        '''
        Gets the access control list for the node

        :param nodename: The Node's nodename
        :type nodename: string
        :return: The acl for the node
        :rtype: hash
        '''
        return super(Nodes, cls).get_acl(nodename)

    @classmethod
    def set_permission(cls, nodename, permission, actors):
        '''
        Grants the specified actors a permission on the node. Chef only
        supports setting one permission at a time.

        The actors hash should be in the form::

            {
                'actors': <list of usernames>
                'groups': <list of groupnames>
            }

        :param nodename: The Node's nodename
        :param permission: One of: create, read, update, delete, grant
        :param actors: The set of actors to grant this permission to
        :type nodename: string
        :type permission: string
        :type actors: hash
        :return: empty hash
        :type: hash
        '''
        return super(Nodes, cls).set_permission(nodename, permission, actors)



'''Databdg provides the methods for chef databags'''

from .base import Base
from pastry.pastry_client import PastryClient


class Databags(Base):
    '''
    Provides methods for interacting with chef cookbooks
    '''
    _base_url = '/organizations/%(org)s/data'

    @classmethod
    def index(cls):
        '''
        Fetches all of the cookbooks (and versions) on the chef server

        :return: All the chef cookbooks and versions
        :rtype: hash
        '''
        return super(Databags, cls).index()

    @classmethod
    def get_item(cls, databagid, itemid):
        '''
        Fetch a databag item from the chef server

        :param databagid: The databag's name
        :type databagid: string
        :param itemid: The item name
        :type itemid: string
        :return: The databag item
        :rtype: hash
        '''
        return PastryClient.call('%s/%s/%s' % (cls.base_url(), databagid, itemid))

    @classmethod
    def create_item(cls, databagid, item):
        '''
        Create a databag item on the chef server

        :param databagid: The databag's name
        :type databagid: string
        :param item: The item hash
        :type item: hash
        :return: The databag item
        :rtype: hash
        '''
        return PastryClient.call('%s/%s' % (cls.base_url(), databagid), method='POST', data=item)

    @classmethod
    def delete_item(cls, databagid, itemid):
        '''
        Delete a databag item from the chef server

        :param databagid: The databag's name
        :type databagid: string
        :param itemid: The item name
        :type itemid: string
        :return: The databag item
        :rtype: hash
        '''
        return PastryClient.call('%s/%s/%s' % (cls.base_url(), databagid, itemid), method='DELETE')


    @classmethod
    def update_item(cls, databagid, itemid, item):
        '''
        Delete a databag item from the chef server

        :param databagid: The databag's name
        :type databagid: string
        :param itemid: The item name
        :type itemid: string
        :param item: The item hash
        :type item: hash
        :return: The databag item
        :rtype: hash
        '''
        return PastryClient.call('%s/%s/%s' % (cls.base_url(), databagid, itemid), method='PUT', data=item)



import mock
import unittest

from pastry.resources.environments import Environments


class EnvironmentsTestCase(unittest.TestCase):

    @mock.patch('pastry.resources.environments.Base.index',
        return_value='index')
    def test_index(self, base):
        self.assertEqual(Environments.index(), 'index')

    @mock.patch('pastry.resources.environments.Base.get',
        return_value='environment')
    def test_get(self, base):
        self.assertEqual(Environments.get('environmentname'), 'environment')

    @mock.patch('pastry.resources.environments.Base.create',
        return_value='environment')
    def test_create(self, base):
        self.assertEqual(Environments.create({}), 'environment')

    @mock.patch('pastry.resources.environments.Base.update',
        return_value='environment')
    def test_update(self, base):
        self.assertEqual(Environments.update('environment', {}), 'environment')

    @mock.patch('pastry.resources.environments.Base.delete',
        return_value='environment')
    def test_delete(self, base):
        self.assertEqual(Environments.delete('environment'), 'environment')

    @mock.patch('pastry.resources.environments.Base.exists',
        return_value=True)
    def test_exists(self, base):
        self.assertEqual(Environments.exists('environment'), True)
